# libsyd: SydB☮x /dev/syd interface implementation

[![SydB☮x](https://git.sr.ht/~alip/syd/blob/main/data/syd.png)](https://todo.sr.ht/~alip/syd)
[![GNU](https://web.archive.org/web/20221222061733if_/https://dev.exherbo.org/~alip/images/gnu.png)](https://www.gnu.org/philosophy/philosophy.html)
[![Linux](https://chesswob.org/jja/tux.png)](https://www.kernel.org/category/about.html)
[![Exherbo](https://web.archive.org/web/20230518155203if_/https://dev.exherbo.org/~alip/images/zebrapig.png)](https://www.exherbolinux.org/docs/gettingstarted.html)
[![musl libc](https://www.chesswob.org/jja/musl-inside.png)](https://www.musl-libc.org/)
[![libsecc☮mp](https://web.archive.org/web/20221222061720if_/https://dev.exherbo.org/~alip/images/libseccomp.png)](https://github.com/seccomp/libseccomp)
[![Paludis](http://paludis.exherbolinux.org/paludis_270.png)](https://paludis.exherbolinux.org)

`libsyd` is a comprehensive C library designed for interfacing with the
SydB☮x environment. It offers functionalities for managing sandbox
states, and facilitating runtime configuration and interaction with the
Sydb☮x sandboxing environment.

[`syd.py`](https://git.sr.ht/~alip/syd/blob/main/lib/src/syd.py) is a
Python module that uses `ctypes` to use `libsyd`.

- Sydb☮x Manual: https://git.sr.ht/~alip/syd
- LibSyd API Documentation: https://libsyd.exherbolinux.org
- Syd.Py API Documentation: https://pysyd.exherbolinux.org
- Syd.Pm API Documentation: https://plsyd.exherbolinux.org
- Syd.Rb API Documentation: https://rbsyd.exherbolinux.org
- Syd.El API Documentation: https://sydel.exherbolinux.org

## Features

- Interact with the `/dev/syd` device for sandbox management.
- Straightforward integration with C and Python applications.

## Prerequisites

- Rust (1.56.1 or later)
- Python 3.x (if using Python bindings)
- Perl, and [`FFI::Platypus`](https://metacpan.org/dist/FFI-Platypus) (if using Perl bindings)
- Ruby, and [`ffi`](https://github.com/ffi/ffi) gem (if using Ruby bindings)
- Emacs (if using `syd.el`)

## Building and Installing from Source

To build and install `libsyd` from source, follow these steps:

1. Clone the repository:
   ```bash
   git clone https://git.sr.ht/~alip/syd
   cd lib
   ```
2. Use the Makefile which utilizes Cargo to build
   ```bash
   make
   make test
   ```
3. Switch to root and install
   ```
   sudo make install
   ```

## BUGS

```
Hey you, out there beyond the wall,
Breaking bottles in the hall,
Can you help me?
```

Report bugs to ___SydB☮x___'s bug tracker at <https://todo.sr.ht/~alip/syd/>:
1. Always **be polite**, respectful, and kind:
   <https://css-tricks.com/open-source-etiquette-guidebook/>
2. Keep your final change as **small and neat** as possible:
   <https://tirania.org/blog/archive/2010/Dec-31.html>
3. Attaching poems with the bug report encourages consideration tremendously.

## COPYRIGHT

- **SPDX-License-Identifier:** *LGPL-3.0-or-later*
- **Copyright ©** 2023 Ali Polatel <alip@chesswob.org>

ChangeLog
=========

# 3.0.7

- go: Add new function `KillMem`.
- lisp: Add new function `syd-kill-mem`.
- perl: Add new function `kill_mem`.
- python: Add new function `kill_mem`.
- ruby: Add new function: `kill_mem`.
- Implement new function `syd_kill_mem`.

# 3.0.6

- go: Add new function `KillPid`.
- lisp: Add new function `syd-kill-pid`.
- perl: Add new function `kill_pid`.
- python: Add new function `kill_pid`.
- ruby: Add new function: `kill_pid`.
- Implement new function `syd_kill_pid`.
- Set MSRV to 1.56.1 as determined by `cargo-msrv`.
- Correct git repository in `Cargo.toml`.

# 3.0.5

- ruby: Add missing functions `Syd.deny_read_del` and `Syd.deny_read_rem`.
- lisp: fix various warnings during byte-compiling with recent Emacs.
- ruby: Add example demonstrating usage.

# 3.0.4

- ruby: Initial Ruby bindings using `ffi` gem.
- lisp: Add new function `syd-info`.
- lisp: Add new function `syd-filter-mem` and `syd-filter-pid`.
- go: Add new functions `FilterMem` and `FilterPid`.
- perl: Add new functions `filter_mem` and `filter_pid`.
- python: Add new functions `filter_mem` and `filter_pid`.
- Implement new functions `syd_filter_mem` and `syd_filter_pid`.

# 3.0.3

- Add `syd.el`, SydB☮x interface for Emacs Lisp
- use **itoa** crate to make `syd_pid_max` function simpler and more
  efficient.
- add examples demonstrating library usage

# 3.0.2

- Fix tests on musl systems
- Drop dependency on the **nix** crate
- Add additional safety checks that `/dev/syd` is the correct file
- Fix a segfault that happens on musl systems

# 3.0.1

- go: initial libsyd Go bindings

# 3.0.0

- Define `LockState` enum and `LOCK_OFF`, `LOCK_EXEC` and `LOCK_ON`
  constants. These were already defined for perl, and python so this
  makes the C library compatible.
- use **itoa** crate to make `syd_load` function simpler and more
  efficient.
- python: Implement the new function **load**.
- perl: Implement the new function **load**.
- Implement the new function **syd\_load**.
- python: Implement new functions **panic** and **reset**.
- perl: Implement new functions **panic** and **reset**.
- Implement new functions **syd\_panic()** and **syd\_reset()**.
- perl: fix argument handling of the `exec` function
- perl: new constant `$syd::LIBSYD_PATH` points to the path of libsyd.so.
- Start using `nix` to call `lstat` rather than Rust std.
- Define `__all__` for `pysyd` such that `from syd import *` becomes
  practical.

# 3.0.0-alpha.6

- Initial Perl bindings

# 3.0.0-alpha.5

- Fix tests

# 3.0.0-alpha.4

- Drop cbindgen dependency, include `syd.h`

# 3.0.0-alpha.3

- Add build instructions to the readme

# 3.0.0-alpha.2

- Fix a problem with Doxygen API documentation generation

# 3.0.0-alpha.1

- Initial implementation of libsyd & syd.py
